'use strict';
const Browserify = require('browserify');
const BrowserSync = require('browser-sync');
const Watchify = require('watchify');
const Gulp = require('gulp');
const GulpUtil = require('gulp-util');
const VinylSourceStream = require('vinyl-source-stream');
const PrettyHrtime = require('pretty-hrtime');
const Path = require('path');
const _ = require('lodash');
const Merge2 = require('merge2');
const Buffer = require('vinyl-buffer');

const TasksUtils = require('../util/utils');
const Plugins = require('../util/plugins');
const Config = TasksUtils.gulpConfig;

const internals = {};

// Browserify has issues with relative paths, we make them all absolute
internals.getAbsolutePath = function (path) {
  return Path.normalize(Path.join(process.cwd(), path));
};

internals.browserifyBundle = function(bundleConfig) {

  bundleConfig = _.cloneDeep(bundleConfig);

  bundleConfig.entries = bundleConfig.entries.map(internals.getAbsolutePath);

  if (TasksUtils.isDev) {
    // Add watchify args and debug (sourcemaps) option
    _.extend(bundleConfig, {
      // watchify.args
      cache: {}, packageCache: {},

      /// source maps
      debug: true
    });
    // A watchify require/external bug that prevents proper recompiling,
    // so (for now) we'll ignore these options during development. Running
    // `gulp browserify` directly will properly require and externalize.
    bundleConfig = _.omit(bundleConfig, ['external', 'require']);
  }

  let b = Browserify(bundleConfig);

  const bundle = function() {
    const startTime = process.hrtime();
    GulpUtil.log(`Bundling ${GulpUtil.colors.green(bundleConfig.outputName)}...`);

    const browserifyStream = b
        .bundle()
        .pipe(VinylSourceStream(bundleConfig.outputName))
        // gulp-concat does not support streams so we use vinyl-buffer to convert the
        // bundle stream to a buffer first
        // https://github.com/wearefractal/gulp-concat/issues/38
        .pipe(Buffer());

    const concatStream = [];

    if (bundleConfig.prepend) {
      const prependAbsPaths = bundleConfig.prepend.map(internals.getAbsolutePath);
      concatStream.push(Gulp.src(prependAbsPaths, { base: '.' }));
    }

    concatStream.push(browserifyStream);

    if (bundleConfig.append) {
      const appendAbsPaths = bundleConfig.append.map(internals.getAbsolutePath);
      concatStream.push(Gulp.src(appendAbsPaths, { base: '.' }));
    }

    return Merge2.apply(null, concatStream)
        .pipe(Plugins.plumber(TasksUtils.streamErrorHandler))
        .pipe(Plugins.sourcemaps.init({loadMaps: true}))
        .pipe(Plugins.concat(bundleConfig.outputName))
        .pipe(Plugins.sourcemaps.write('./sourcemaps'))
        .pipe(Gulp.dest(bundleConfig.dest))
        .pipe(BrowserSync.reload({
          stream: true
        }))
        .on('end', () => {
          const taskTime = process.hrtime(startTime);
          const prettyTime = PrettyHrtime(taskTime);
          GulpUtil.log(`Bundled ${GulpUtil.colors.green(bundleConfig.outputName)} in ${GulpUtil.colors.magenta(prettyTime)}`);
        });
  };

  if(TasksUtils.isDev) {
    let changedFiles = []; // List of files changed since last re-bundle

    // We use debounce to make sure we'll only re-bundle once when multiple files change
    const debouncedBundle = _.debounce(() => {
      const formattedPaths = changedFiles.map(function(path) {
        return '\t* ' + GulpUtil.colors.cyan(path);
      }).join('\n');
      GulpUtil.log('The following files changed:\n' + formattedPaths);
      changedFiles = [];
      bundle();
    }, 300);

    b = Watchify(b);
    b.on('update', (paths) => {
      changedFiles = changedFiles.concat(paths);
      debouncedBundle();
    });

    if (bundleConfig.prepend || bundleConfig.append) {
      const files = [].concat(
          bundleConfig.prepend || [],
          bundleConfig.append || []
      );
      const watch = Gulp.watch(files, (ev) => {
        changedFiles.push(ev.path);
        debouncedBundle();
      });
    }

    GulpUtil.log('Watching files required by', GulpUtil.colors.yellow(bundleConfig.outputName));
  } else {
    // Sort out shared dependencies.
    // b.require exposes modules externally
    if(bundleConfig.require) b.require(bundleConfig.require);
    // b.external excludes modules from the bundle, and expects
    // they'll be available externally
    if(bundleConfig.external) b.external(bundleConfig.external);
  }

  return bundle();
};

exports = module.exports = {};

exports.browserifyTask = function() {
  const bundleStreams = _.map(Config.build.browserify.bundleConfigs, internals.browserifyBundle);
  return Merge2.apply(null, bundleStreams);
};

Gulp.task('build-browserify', function() {
  // do nothing if there are no bundles
  if (!Config.build.browserify ||
    !Config.build.browserify.bundleConfigs ||
    !Config.build.browserify.bundleConfigs.length) {
    return;
  }
  return exports.browserifyTask();
});




