'use strict';
const Gulp = require('gulp');
const GulpUtil = require('gulp-util');
const RunSequence = require('run-sequence');
const _ = require('lodash');

const Plugins = require('../util/plugins');
const TasksUtils = require('../util/utils');
const Config = TasksUtils.gulpConfig;


Gulp.task('deploy', function(callback) {
  RunSequence('deploy-s3',
    callback);
});

// Send the static files to S3
Gulp.task('deploy-s3', function() {

  // create a new publisher using S3 options
  // http://docs.aws.amazon.com/AWSJavaScriptSDK/latest/AWS/S3.html#constructor-property
  const publisher = Plugins.awspublish.create({
    params: {
      Bucket: Config.deploy.s3.s3Bucket
    },
    region: Config.deploy.s3.s3Region
  });

  let routes = [
    // The sitemap is generated from metalsmith so it will be gzipped in the production version
    // We'll let it 5 minutes of cache
    {
      routeMatcher: /^sitemap.xml$/,
      headers: {
        ContentEncoding: 'gzip'
      }
    },

    // Css and Js have cache busters so they are cached indefinitely and are gzipped
    {
      routeMatcher: /^(?:.+)\.(?:js|css|svg|ttf)$/,
      cacheTime: 315360000, // 10 years
      headers: {
        ContentEncoding: 'gzip'
      }
    },

    // Image files have a cache buster, but are not gzipped
    {
      routeMatcher: /^(?:.+)\.(?:png|gif|jpg|jpeg)$/,
      cacheTime: 315360000 // 10 years
    },

    // Html files have no cache buster at all, we'll define their caching behavior in cloudfront itself
    {
      routeMatcher: /^(?:.+)\.(?:html)$/,
      headers: {
        ContentEncoding: 'gzip'
      }
    },

    // pass-through for anything that wasn't matched by routes above,
    // to be uploaded with default options
    {
      routeMatcher: /^.+$/,
      key: "$&"
    }
  ];

  if (Config.deploy.awsPublish && Config.deploy.awsPublish.extraRoutes) {
    const extraRoutes = Config.deploy.awsPublish.extraRoutes;
    routes = extraRoutes.concat(routes);

    // The cache mechanism of gulp-awspublish does not handle headers
    // So if a route has a header, such as WebsiteRedirectLocation, it will not update as expected if changed
    // The only way around it is to clear the cache for such routes
    // TODO remove when issue 73 is resolved https://github.com/pgherveou/gulp-awspublish/issues/73
    let cachedFiles = _.keys(publisher._cache);
    extraRoutes.forEach((route) => {
      if (route.headers) {
        _.remove(cachedFiles, (path) => {
          return route.routeMatcher.test(path);
        }).forEach((removedPath) => {
          GulpUtil.log(`Ignoring publish cache for ${removedPath} because it has headers`);
          delete publisher._cache[removedPath];
        });
      }
    });
  }

  return Gulp.src(['**/*'], { cwd: Config.production.dir })
    .pipe(Plugins.plumber(TasksUtils.streamErrorHandler))

    .pipe(Plugins.awspublishRouter({
      cache: {
        // html files should not be cached
      },
      routes: routes
    }))

    .pipe(publisher.publish())

    // create a cache file to speed up consecutive uploads
    .pipe(publisher.cache())

    // print upload updates to console
    .pipe(Plugins.awspublish.reporter());
});