﻿'use strict';
const Gulp = require('gulp');
const Del = require('del');
const RunSequence = require('run-sequence');
const Joi = require('joi');
const Path = require('path');
const _ = require('lodash');
const MetalsmithGzip = require('metalsmith-gzip');

const Plugins = require('../util/plugins');
const Metalsmith = require('../util/metalsmith');
const StaticAssets = require('../util/staticAssets');
const TasksUtils = require('../util/utils');
const Config = TasksUtils.gulpConfig;


// Prepare the files for uploading to the production server:
//    Gzip the static files (js, css)
//    Copy images
//    Change the links in the markup to point to the paths where the files would be
//    uploaded to in Amazon S3
Gulp.task('production', function(callback) {
  RunSequence('production-clean',
    ['production-copy-gzip', 'production-copy'],
    'production-static-site',
    callback);
});

Gulp.task('production-clean', function() {
  return Del([
    Config.production.dir
  ]);
});

Gulp.task('production-copy-gzip', function() {
  return Gulp.src(Config.production.copyGzip.src, { cwd: Config.compile.dir })
    .pipe(Plugins.plumber(TasksUtils.streamErrorHandler))
    .pipe(Plugins.gzip(Config.production.copyGzip.gzipOptions))
    .pipe(Gulp.dest(Config.production.dir))
    .pipe(Plugins.filesize());
});

Gulp.task('production-copy', function() {
  return Gulp.src(Config.production.copy.src, { cwd: Config.compile.dir })
    .pipe(Plugins.plumber(TasksUtils.streamErrorHandler))
    .pipe(Gulp.dest(Config.production.dir));
});

Gulp.task('production-static-site', function(done) {
  Joi.assert(Config.deploy.s3, Joi.object().keys({
    s3Region: Joi.string().valid(
      'us-east-1',
      'us-west-2',
      'us-west-1',
      'eu-west-1',
      'eu-central-1',
      'ap-southeast-1',
      'ap-southeast-2',
      'ap-northeast-1',
      'sa-east-1').required(),
    s3Bucket: Joi.string().required()
  }));

  Joi.assert(Config.production, Joi.object().keys({
    staticUrl: Joi.alternatives().try(
      Joi.any().valid(''),
      Joi.string()
        .uri()
        .regex(/[^\/]$/, 'not end with a slash')
        .required()
    ).description('The URL to add as prefix for static assets')
  }).unknown(true));

  Config.production.getMarkupData().then(function(markupData) {
    markupData = _.assign(markupData, {
      baseUrl: '',
      staticUrl: (url) => {
        return `${Config.production.staticUrl}/${StaticAssets.filenameWithRev(url)}`;
      }
    });

    Metalsmith.build(markupData)
      .use(MetalsmithGzip(Config.production.metalsmithGzip))
      .destination(Config.production.dir)
      .build(function(err) {
        if (err) {
          TasksUtils.errorHandler(err);
        }
        done(err);
      });

  }).catch(done);
});
